/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "MBMesh.hpp"

namespace parncl{

MBMesh::MBMesh(moab::Interface *mb_gcore, MPI_Comm mb_comm, moab::ParallelComm *mb_pcomm, MeshFInfo *mesh_finfo, const std::string &part_method):mb_gcore_(mb_gcore), mb_comm_(mb_comm), mb_pcomm_(mb_pcomm), mesh_finfo_(mesh_finfo), part_method_(part_method), ref_cnt_(0)
{
	// FIXME: Will we allow other MPI error handlers ?
	MPI_Comm_rank(mb_comm_, &mb_rank_);
}

MBMesh::~MBMesh()
{
//	mb_gcore_->clear_meshset(&mb_mset_, 1);
	if(mesh_finfo_){
		delete mesh_finfo_;
	}
}

void MBMesh::add_mesh_val(ParCALMeshVal *mesh_val)
{
	mesh_vals_.push_back(mesh_val);
}

void MBMesh::del_mesh_val(ParCALMeshVal *mesh_val)
{
	std::list<ParCALMeshVal *>::iterator iter = std::find(mesh_vals_.begin(), mesh_vals_.end(), mesh_val);
	if(iter != mesh_vals_.end()){
		mesh_vals_.erase(iter);
	}
}

MeshFInfo &MBMesh::get_mesh_finfo(void ) const
{
	return (*mesh_finfo_);
}

moab::Interface *const MBMesh::get_mb_core(void ) const
{
	return mb_gcore_;
}

moab::EntityHandle MBMesh::get_mb_mset(void ) const
{
	return mesh_finfo_->get_mb_mset();
}

MPI_Comm MBMesh::get_mpi_comm(void ) const
{
	return mb_comm_;
}

int MBMesh::get_mpi_rank(void ) const
{
	return mb_rank_;
}

std::string MBMesh::get_part_method(void ) const
{
	return part_method_;
}

// FIXME: Get rid of these public ref count controllers - use a smart ptr

void MBMesh::inc_ref_cnt(void )
{
	ref_cnt_++;
}

void MBMesh::dec_ref_cnt(void )
{
	ref_cnt_--;
}

int MBMesh::get_ref_cnt(void ) const
{
	return ref_cnt_;
}

void MBMesh::set_ref_cnt(int cnt)
{
	ref_cnt_ = cnt;
}

} // namespace parncl
