/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "PNclMDUtils.hpp"
#include <cstring>

namespace parncl{

NclSelectionRecord *PNclSelOfSel(const NclSelectionRecord *orig_sel, NclSelectionRecord *new_sel)
{
	for(int i=0; (i<new_sel->n_entries) && (i<orig_sel->n_entries); i++){
		if(orig_sel->selection[i].sel_type == Ncl_SUB_ALL){
			// No change needed in new_sel, leave it as it is
			continue;
		}
		else if(new_sel->selection[i].sel_type == Ncl_SUB_ALL){
			// Just copy the orig selector
			PNclSelCopy(new_sel, orig_sel);
		}
		else if(new_sel->selection[i].sel_type == Ncl_SUBSCR){
			if(orig_sel->selection[i].sel_type == Ncl_SUBSCR){
				new_sel->selection[i].u.sub.start += orig_sel->selection[i].u.sub.start;
				new_sel->selection[i].u.sub.finish += orig_sel->selection[i].u.sub.start;
				new_sel->selection[i].u.sub.stride *= orig_sel->selection[i].u.sub.stride;
			}
			else if(orig_sel->selection[i].sel_type == Ncl_VECSUBSCR){
				size_t n_ind = orig_sel->selection[i].u.vec.n_ind;
				size_t n_new_ind = (new_sel->selection[i].u.sub.finish - new_sel->selection[i].u.sub.start)/(new_sel->selection[i].u.sub.stride) + 1; 

				new_sel->selection[i].sel_type = Ncl_VECSUBSCR;
				new_sel->selection[i].u.vec.n_ind = n_new_ind;
				new_sel->selection[i].u.vec.ind = (long *)NclCalloc(n_new_ind, sizeof(long));
				for(size_t j=new_sel->selection[i].u.sub.start, k=0; (j<=new_sel->selection[i].u.sub.finish) && (k<n_new_ind); j+=new_sel->selection[i].u.sub.stride, k++){
					if(j >= n_ind){
						NhlPError(NhlFATAL, NhlEUNKNOWN, "Invalid selection");
						return NULL;
					}
					new_sel->selection[i].u.vec.ind[k] = orig_sel->selection[i].u.vec.ind[j];
				}
				// FIXME: Verify that this is correct
				new_sel->selection[i].u.vec.min = new_sel->selection[i].u.vec.ind[0];
				new_sel->selection[i].u.vec.max = new_sel->selection[i].u.vec.ind[n_new_ind - 1];
			}
			else{
				NhlPError(NhlFATAL, NhlEUNKNOWN, "Unknown selector type");
				return NULL;
			}
		}
		else if(new_sel->selection[i].sel_type == Ncl_VECSUBSCR){
			size_t n_ind = new_sel->selection[i].u.vec.n_ind;
			if(orig_sel->selection[i].sel_type == Ncl_SUBSCR){
				for(size_t j=0; j<n_ind; j++){
					long idx = new_sel->selection[i].u.vec.ind[j];
					new_sel->selection[i].u.vec.ind[j] = orig_sel->selection[i].u.sub.start + idx * orig_sel->selection[i].u.sub.stride;
				}
			}
			else if(orig_sel->selection[i].sel_type == Ncl_VECSUBSCR){
				for(size_t j=0; j<n_ind; j++){
					long idx = new_sel->selection[i].u.vec.ind[j];
					if(idx >= orig_sel->selection[i].u.vec.max){
						NhlPError(NhlFATAL, NhlEUNKNOWN, "Invalid selection");
						return NULL;
					}
					new_sel->selection[i].u.vec.ind[j] = orig_sel->selection[i].u.vec.ind[idx];
				}
			}
			else{
				NhlPError(NhlFATAL, NhlEUNKNOWN, "Unknown selector type");
				return NULL;
			}
			new_sel->selection[i].u.vec.min = new_sel->selection[i].u.vec.ind[0];
			new_sel->selection[i].u.vec.max = new_sel->selection[i].u.vec.ind[n_ind - 1];
		}
		else{
			NhlPError(NhlFATAL, NhlEUNKNOWN, "Unknown selector type");
			return NULL;
		}
	}
	return new_sel;
}

NclSelectionRecord *PNclSelCopy(NclSelectionRecord *to_sel, const NclSelectionRecord *from_sel, const size_t from_sel_index)
{
	if(!from_sel || (from_sel->n_entries == 0)){
		return to_sel;
	}

	*to_sel = *from_sel;
	to_sel->n_entries = 0;
	for(size_t i=0, j=from_sel_index; j<from_sel->n_entries; i++, j++){
		if(i != j){
			to_sel->selection[i] = from_sel->selection[j];	
			to_sel->selection[i].dim_num = i;
		}
		if(from_sel->selection[j].sel_type == Ncl_VECSUBSCR){
			/* FIXME: We currently ignore cases where to_sel was valid. In that
			 * case we need to explicitly copy all elements instead of doing the
			 * easy *to_sel = *from-sel
			 */
			size_t n_ind = from_sel->selection[j].u.vec.n_ind;
			/*
			if(to_sel->selection[i].sel_type != Ncl_VECSUBSCR){
				to_sel->selection[i].u.vec.ind = (long *)NclCalloc(n_ind, sizeof(long));
			}
			else{
				if(to_sel->selection[i].u.vec.n_ind != n_ind){
					NclFree(to_sel->selection[i].u.vec.ind);
					to_sel->selection[i].u.vec.ind = (long *)NclCalloc(n_ind, sizeof(long));
					to_sel->selection[i].u.vec.n_ind = n_ind;
				}
			}
			*/
			to_sel->selection[i].u.vec.ind = (long *)NclCalloc(n_ind, sizeof(long));
			if(to_sel->selection[i].u.vec.ind == NULL){
				NhlPError(NhlFATAL, NhlEUNKNOWN, "Unable to allocate mem for copying sel");
				return NULL;
			}
			memcpy(to_sel->selection[i].u.vec.ind, from_sel->selection[j].u.vec.ind, n_ind * sizeof(long));
		}
		to_sel->n_entries++;
	}
	return to_sel;
}

NclSelectionRecord *PNclSelCopy(NclSelectionRecord *to_sel, const NclSelectionRecord *from_sel)
{
	return PNclSelCopy(to_sel, from_sel, 0);
}

void PNclGetSelSz(NclSelectionRecord *sel, std::vector<size_t> &sel_sz)
{
	size_t n_entries = sel->n_entries;
	if(n_entries != sel_sz.size()){
		sel_sz.resize(n_entries);
	}
	for(size_t i=0; i<n_entries; i++){
		switch(sel->selection[i].sel_type){
			case Ncl_SUB_ALL:
			case Ncl_SUBSCR:
				sel_sz[i] = (sel->selection[i].u.sub.finish - sel->selection[i].u.sub.start)/(sel->selection[i].u.sub.stride) + 1;
				break;
			case Ncl_VECSUBSCR:
				sel_sz[i] = sel->selection[i].u.vec.n_ind;
				break;
			default:
				NhlPError(NhlFATAL, NhlEUNKNOWN, "Unknown selector type");
		}
	}
}

} // namespace parncl

