/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef ParCALVarSHnd_hpp
#define ParCALVarSHnd_hpp
#include <vector>
#include "ParCALCore.hpp"
extern "C"{
#include "ParCALFuncs.h"
}
#include "MBMesh.hpp"
#include "ParCALMeshVal.hpp"
#include "PNclMBUtils.hpp"

namespace parncl{
	const int PARCALVAR_NCHUNK_ALL = -1;
	const int PARNCL_INVALID_CALLER_LEVEL = -1;
	class ParCALVarSHnd{
		public:
			/* Create Var from mesh val sptr */
			ParCALVarSHnd(ParCALMeshValSPtr &sptr);
			/* Create Var from mesh val sptr, also store a sel for the val */
			ParCALVarSHnd(ParCALMeshValSPtr &sptr, NclSelectionRecord *sel);
			/* Dup a var */
			ParCALVarSHnd(const ParCALVarSHnd &var);
			/* Slice a var */
			ParCALVarSHnd(const ParCALVarSHnd &var, NclSelectionRecord *sel);

			/*	
			 *	FIXME: Handle calls without upper layer knowing num chunks
			 *	e.g.: Use an iterator, get_next_chunk()
			 */
			int get_num_val_chunks(void );
			void *get_val(NclSelectionRecord *sel, size_t req_chunk, ParCALValType val_type, ParCALAccessMode mode, size_t &sz);
			void delete_val(int del_chunk);
			pargal::pcvar *get_pcvar(void );
			size_t get_num_val_levs(void );
			pargal::fileinfo &get_pcal_finfo(void );
			MBMesh *get_mesh(void );
			nc_type get_data_type(void );
			void *get_missing_val(void );
			// FIXME: Avoid exposing the mesh val loc 
			ParCALMeshVal::MeshValLoc get_mesh_val_loc(void );
			void get_sel(NclSelectionRecord *sel);
			void set_declone_caller_level(int caller_level);
			int get_declone_caller_level(void ) const;
			void declone(void );
			~ParCALVarSHnd();
		private:
			ParCALVarSHnd();

			ParCALMeshValSPtr sptr_;
			// FIXME: We don't cache lvals for now
			std::vector<int > cached_lval_chunks_;
			NclSelectionRecord *sel_;
			int declone_caller_level_;
	};
} // namespace parncl

#endif // ParCALVarSHnd_hpp
